/* List */
/* $Id: list.c,v 0.5 2003/11/08 19:17:13 bsp Exp $ */
/* ... */

#include <stdlib.h>
#include <string.h>
#include "ptxlist.h"

ptxlist *list_error_ptr;

ptxlist *init_list (void *data, size_t size)
{
	ptxlist *l;
	char *d;
	
	l = calloc (1, sizeof (ptxlist));
	if (!l) 
	{
		list_error_ptr = NULL;
		return NULL;
	}
	
	d = calloc (size, sizeof (char));
	if (!d)
	{
		free (l);
		list_error_ptr = NULL;
		return NULL;
	}
	
	memcpy (d, data, size);
	
	l->data = (void *) d;
	return l;
}

void goto_head (ptxlist **l)
{
	if (!l || !*l) return;
	while ((*l)->prev) *l = (*l)->prev;
}

void goto_tail (ptxlist **l)
{
	if (!l || !*l) return;
	while ((*l)->next) *l = (*l)->next;
}

void goto_next (ptxlist **l)
{
	if (!l || !*l) return;
	if ((*l)->next) *l = (*l)->next;
}

void goto_prev (ptxlist **l)
{
	if (!l || !*l) return;
	if ((*l)->prev) *l = (*l)->prev;
}

int is_last (ptxlist *l)
{
	if (!l) return -1;
	if (l->next) return 0;
	else return 1;
}

int is_empty (ptxlist *l)
{
	if (!l) return -1;
	if ((!l->next) && (!l->prev)) return 1;
	else return 0;
}

int is_first (ptxlist *l)
{
	if (!l) return -1;
	if (l->prev) return 0;
	else return 1;
}

ptxlist *add_to_list (ptxlist *l, void *data, size_t size)
{
	ptxlist *tmp;
	char *d;
	
	tmp = calloc (1, sizeof (ptxlist));
	if (!tmp) 
	{
		list_error_ptr = l;
		return NULL;
	}
	
	d = calloc (size, sizeof (char));
	if (!d)
	{
		free (tmp);
		list_error_ptr = l;
		return NULL;
	}
	
	memcpy (d, data, size);
	
	tmp->data = d;
	
	if (!l) return tmp;
	tmp->next = l->next;
	tmp->prev = l;
	l->next = tmp;
	
	return tmp;
}

ptxlist *add_to_head (ptxlist *l, void *data, size_t size)
{
	ptxlist *tmp;
	char *d;
	
	goto_head (&l);
	
	tmp = calloc (1, sizeof (ptxlist));
	if (!tmp) 
	{
		list_error_ptr = l;
		return NULL;
	}
	
	d = calloc (size, sizeof (char));
	if (!d)
	{
		free (tmp);
		list_error_ptr = l;
		return NULL;
	}
	
	memcpy (d, data, size);
	
	tmp->data = d;	
	tmp->next = l;
	if (l) l->next = tmp;
	
	return tmp;
}


ptxlist *add_to_tail (ptxlist *l, void *data, size_t size)
{
	goto_tail (&l);
	
	return add_to_list (l, data, size); 
}

void del_list (ptxlist *l)
{
	ptxlist *tmp;
	goto_head (&l);
	
	while (l)
	{
		tmp = l->next;
		free (l->data);
		free (l);
		l = tmp;
	}
}

ptxlist *del_from_list (ptxlist *l)
{
	ptxlist *tmp;
	
	if (l->next) tmp = l->next;
	else tmp = l->prev;
	
	free (l->data);
	free (l);
	
	return tmp;
}

ptxlist *find_in_list (ptxlist *l, int (*fnct) (void *, void *), void *test)
{
	for (goto_head (&l); l;l = l->next)
	{
		if (fnct (l->data, test)) break;
	}
	return l;
}

void *get_list_element_data (ptxlist *l)
{
	if (!l) return NULL;
	return l->data;
}
